#include <PalmOS.h>
#include <SystemMgr.h>

#define PLUGIN_TRAP(noTrap)

#include "plugin.h"
#include "PlugLib.h"
#include "datasource.h"
#include "grid.h"

#if defined(__GNUC__)
Err
start (UInt16 refnum, SysLibTblEntryPtr entryP) {
#else
Err
__Startup__ (UInt16 refnum, SysLibTblEntryPtr entryP) {
#endif
  extern void *jmptable ();
  entryP->dispatchTblP = (void *) jmptable;
  entryP->globalsP = NULL;
  return 0;
  }

PlugLib_globals * GlobalsLock(UInt16 refnum)
{
    SysLibTblEntryPtr entryP = SysLibTblEntry (refnum);
    if (entryP->globalsP)
        return MemHandleLock(entryP->globalsP);
    else
        return NULL;
}

void GlobalsUnlock(PlugLib_globals * gP)
{
    MemPtrUnlock(gP);
}

/* Four standard functions */
/* open the library and anything else that need to be done at startup */
Err PlugLib_open(UInt16 refnum)
{
    PlugLib_globals *gl = GlobalsLock(refnum);

    if (!gl) {
        SysLibTblEntryPtr entryP = SysLibTblEntry (refnum);
        /* We need to allocate space for the globals.  */
        entryP->globalsP = MemHandleNew(sizeof(PlugLib_globals));
        gl = MemHandleLock(entryP->globalsP);
        MemSet(gl, sizeof (PlugLib_globals), 0);
        MemPtrSetOwner(gl, 0);
/*
        gl->refcount = 0;
        gl->numActions = 0;
        gl->driver = NULL;
*/
    }

    gl->refcount++;

    MainPlug(gl);
    GlobalsUnlock(gl);
  return 0;
}

/* close the library and anything else that need to be done at shutdown */
Err PlugLib_close(UInt16 refnum, UInt16 * numappsP)
{
    PlugLib_globals *gl = GlobalsLock(refnum);

    if (!gl) {
        /* We're not open! */
        return 1;
    }
  
    /* Clean up.  */
    *numappsP = gl->refcount;
    GlobalsUnlock(gl);
    if (*numappsP == 0) {
        SysLibTblEntryPtr entryP = SysLibTblEntry (refnum);
        MemHandleFree (entryP->globalsP);
        entryP->globalsP = NULL;
    }

    return 0;
}

/* do anything that need to be done when the processor sleeps */
Err PlugLib_sleep(UInt16 refnum)
{
    return 0;
}

/* do anything that need to be done when the processor wakes up */
Err PlugLib_wake(UInt16 refnum)
{
    return 0;
}

UInt32
PlugLib_getVersion(UInt16 refnum)
{
    return (UInt32) PlugLibVersion;
}

void
PlugLib_showAboutBox(UInt16 refnum)
{
    PlugLib_globals *gl = GlobalsLock(refnum);

    if (gl) {
        if (gl->aboutAlertID) {
            FrmAlert(gl->aboutAlertID);
        }
    }
    GlobalsUnlock(gl);
}

DataSourceDriver *
PlugLib_getDriver(UInt16 refnum)
{
    PlugLib_globals *gl = GlobalsLock(refnum);
    DataSourceDriver * driverP = NULL;

    if (gl) {
        if (gl->driver){
            driverP = gl->driver;
        }
    }
    GlobalsUnlock(gl);
    return driverP;
}

void * PlugLib_getAction(UInt16 refnum, UInt16 action)
{
     PlugLib_globals *gl = GlobalsLock(refnum);
     void * function = NULL;

    if (gl) {
        UInt16 i;

        for (i = 0; i < gl->numActions; i++) {
            if (gl->actions[i].actionID == action) {
                function = gl->actions[i].function;
                break;
            }
        }
    }

    GlobalsUnlock(gl);
    return function;
}

GridType * PlugLib_getGrid(UInt16 refnum)
{
    return NULL;
}
