#ifndef __PLUGINTERFACE_H__
#define __PLUGINTERFACE_H__

struct data_source;
struct data_source_driver;
struct grid_data;

/**
 * $Function: MainPlug
 *
 * $Comment: This function must be create (like PilotMain), is the main entry of the plugin
 * $Arguments: plugData     is the data from the main application.
 *                          Some functions need this pointer.
 * $Return: nothing
 **/
extern void MainPlug(void *plugData);

/* Actions functions */
/**
 * $Function: Plug_SetDriver
 *
 * $Comment: set a new databases driver
 * $Arguments: pluginData must be the pointer from MainPlug
 *             driver is the driver structure which is described in "datasource.h"
 * $Return: nothing
 **/
extern void Plug_SetDriver(void *pluginData, struct data_source_driver *driver);

/**
 * $Function: Plug_SetAboutAlert
 *
 * $Comment: set the Alert displayed in the "help/...plugin" menu
 * $Arguments: pluginData must be the pointer from MainPlug
 *             alertID the ID gave in the rcp file
 * $Return: nothing
 **/
extern void Plug_SetAboutAlert(void *pluginData, UInt16 alertID);

/**
 * $Function: Plug_SetAction
 *
 * $Comment: set the functions called by the main application
 * $Arguments: pluginData   must be the pointer from MainPlug
 *             action       the type of action that should launch the function
 *             function     the function to launch
 * $Return: nothing
 **/
typedef enum {
    ActionRunInListViewSelector = 1,
    ActionRunInEditViewSelector,
    ActionRunInChooserSelector,
    ActionRunWhenListViewOpen = 11,
    ActionRunWhenEditViewOpen
} Plug_Action;
typedef void Plug_ActionCallBack (void *pluginData);
extern void Plug_SetAction(void *pluginData, Plug_Action action,
                            Plug_ActionCallBack * function);

/* Data informations functions */
/**
 * $Function: Plug_GetSource
 *
 * $Comment: give the current database source structure
 *           a database must be opened
 * $Arguments: pluginData   must be the pointer from MainPlug
 * $Return: the structure of the current database which is described in "datasource.h"
 **/
extern struct data_source * Plug_GetSource(void *pluginData);

/**
 * $Function: Plug_GetRecord
 *
 * $Comment: give the current record index
 *           a database must be opened and a record be displaid
 * $Arguments: pluginData   must be the pointer from MainPlug
 * $Return: the record index
 **/
extern UInt16 Plug_GetRecord(void *pluginData);

/* Form Action functions */
/* Important:
 * this functions run inside the plugin, the main application is not called
 * the data can be the pointer from the MainPlug (if you want use Plug_GetRecord...)
 * or another pointer (you create a new pointer befor the call and free it after).
 */

typedef Boolean Plug_HandleEventFuncType(EventPtr e, void * data);

/**
 * $Function: Plug_ReturnToForm
 *
 * $Comment: replace the FrmReturnToForm function of the PalmOS API
 * $Arguments:  pluginData   must be the pointer from MainPlug
 *              formID  is the ID to return, if is 0 the events return to the main
 *                      application.
 * $Return: nothing
 **/
extern void Plug_ReturnToForm(void *pluginData, UInt16 formID);

/**
 * $Function: Plug_DoForm
 *
 * $Comment: replace the FrmDoDialog function of the PalmOS API but with a specific handler.
 *           There is an internal event loop.
 * $Arguments:  pluginData   must be the pointer from MainPlug
 *              formID      is the ID of the form give in the rcp file
 *                          (becarfull the ID MUST be not used by the main application
 *                           or another plugin)
 *              handleEvent is the event handler which treate every event on the form.
 *                          The handler must draw form when the frmOpenEvent occurs,
 *                          calls Plug_ReturnToForm to close it-self.
 *                          The frmCloseEvent DOESN'T occur when the form is closed.
 *              data        is a pointer to keep data for the handler.
 * $Return: nothing
 **/
extern void Plug_DoForm(void *pluginData, UInt16 formID, Plug_HandleEventFuncType handleEvent, void * data);

/**
 * $Function: Plug_GotoForm
 *
 * $Comment: replace the FrmGotoForm function of the PalmOS API
 * $Arguments:  pluginData   must be the pointer from MainPlug
 *              formID      is the ID of the form give in the rcp file
 *                          (becarfull the ID MUST be not used by the main application
 *                           or another plugin)
 *              handleEvent is the event handler which treate every event on the form.
 *                          The handler must draw form when the frmOpenEvent occurs,
 *                          calls Plug_ReturnToForm to close it-self.
 *                          The frmCloseEvent DOESN'T occur when the form is closed.
 *              data        is a pointer to keep data for the handler.
 * $Return: nothing
 **/
extern void Plug_GotoForm(void *pluginData, UInt16 formID, Plug_HandleEventFuncType handleEvent, void * data);

#endif/* __PLUGINTERFACE_H__*/
