#ifndef _LEAK_DETECT_H_
#define _LEAK_DETECT_H_

#include "section.h"

#define	 ASSERT(c)         ErrFatalDisplayIf(!(c),#c)

#ifdef DETECT_LEAKS
MemPtr AllocateMemPtrFn(long size, char *file, long line) DEBUGSECT;
Err   DeallocateMemPtrFn(MemPtr addr, char *file, long line) DEBUGSECT;
MemHandle AllocateMemHandleFn(long size, char *file, long line) DEBUGSECT;
Err DeallocateMemHandleFn(MemHandle addr, char *file, long line) DEBUGSECT;
void RecordMemInfoFn( void *addr, char *str, char *file, long line ) DEBUGSECT; 
MemPtr LockMemHandleFn(MemHandle addr, char *file, long line) DEBUGSECT;
Err UnlockMemHandleFn(MemHandle addr, char *file, long line) DEBUGSECT;
Err UnlockMemPtrFn(MemPtr addr, char *file, long line) DEBUGSECT;
void LeakDetect_FinalCallBeforeStoppingApp() DEBUGSECT;

#define AllocateMemPtr(size) AllocateMemPtrFn(size,__FILE__,__LINE__)
#define DeallocateMemPtr(addr) DeallocateMemPtrFn(addr,__FILE__,__LINE__)
#define AllocateMemHandle(size) AllocateMemHandleFn(size,__FILE__,__LINE__)
#define DeallocateMemHandle(addr) DeallocateMemHandleFn(addr,__FILE__,__LINE__)
#define RecordMemInfo(addr,str) RecordMemInfoFn(addr,str,__FILE__,__LINE__)
#define LockMemHandle(addr) LockMemHandleFn(addr,__FILE__,__LINE__)
#define UnlockMemHandle(addr) UnlockMemHandleFn(addr,__FILE__,__LINE__)
#define UnlockMemPtr(addr) UnlockMemPtrFn(addr,__FILE__,__LINE__)

#else /* ! DETECT_LEAKS */
#define AllocateMemPtr(size) ((size > 0)? MemPtrNew(size): NULL)
#define DeallocateMemPtr(addr) MemPtrFree(addr)
#define AllocateMemHandle(size) ((size > 0)? MemHandleNew(size): NULL)
#define DeallocateMemHandle(addr) MemHandleFree(addr)
#define RecordMemInfo(addr,str) {}
#define LockMemHandle(addr) MemHandleLock(addr)
#define UnlockMemHandle(addr) MemHandleUnlock(addr)
#define UnlockMemPtr(addr) MemPtrUnlock(addr)
#endif /* ! DETECT_LEAKS */

#endif 



