/*
 * DB: Plugins manager
 * Copyright (C) 2001-2003: Marc Chalain (marc-chalain@voila.fr)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <PalmOS.h>

#ifdef CONFIG_HANDERA
#include <Vga.h>
#endif

#include "db.h"
#include "datasource.h"
#include "grid.h"
#include "plug.h"
#include "PlugLib.h"
#include "util.h"
#include "enum.h"

typedef void ActionVer1 (void *);

Plugin PluginTable[MAXPLUGIN];

PlugEnvironement Environement;
UInt16 PluginLaunched;

static void Restore(int i) PLUGINSECT;

void Plug_Execute(UInt16 pluginToLaunch, 
                    const PlugEnvironement environement, Switcher switcher)
{
    PlugLib_actionID action;

    switch(environement.screen) {
    case PLUGINTYPE_ABOUTBOX:
        if (PluginTable[pluginToLaunch].dbRef)
            PlugLib_showAboutBox(PluginTable[pluginToLaunch].shLibRef);
    break;
    case PLUGINTYPE_LISTVIEW:
        action = switcher + RunAtListView;
        goto plugin_executeaction;

    case PLUGINTYPE_EDITVIEW:
        action = switcher + RunAtEditView;
        goto plugin_executeaction;

    case PLUGINTYPE_CHOOSER:
        action = switcher + RunAtChooser;
        goto plugin_executeaction;

    case PLUGINTYPE_DATABASEINFO:
        action = RunAtCategoryChanged;
        goto plugin_executeaction;

plugin_executeaction:
        if (PluginTable[pluginToLaunch].dbRef 
                && PlugLib_getVersion(PluginTable[pluginToLaunch].shLibRef) > 1) {
            ActionVer1 * function;
            function = (ActionVer1 *)PlugLib_getAction(PluginTable[pluginToLaunch].shLibRef,
                         action);
            if (function) {
                PlugLib_appliDataV1 data;
                data.source = environement.source;
                data.record = environement.record;
                data.driverlist = DriverList;
#ifdef CONFIG_HANDERA
                if (FeatureSet.Handera)
                    VgaSetScreenMode(screenModeScaleToFit, rotateModeNone);
#endif
                function(&data);
#ifdef CONFIG_HANDERA
                if (FeatureSet.Handera)
                    VgaSetScreenMode(screenMode1To1, rotateModeNone);
#endif
            }
        }
    break;    
    default:
    }
}

Boolean
PluginListHandleEvent( EventPtr event)
{
        FormPtr form;
        ListPtr list;
        static Char *listName[MAXPLUGIN];
        UInt16 i, count = 0;
 
        switch( event->eType)
        {
        case frmOpenEvent:
            form = FrmGetActiveForm(); 
#ifdef CONFIG_HANDERA
            if (FeatureSet.Handera) {
                VgaFormModify(form, vgaFormModify160To240);
                if (!UtilCheckSizeForm(form, 0)) {
                    FrmAlert(alertID_FormTooBig);
                    FrmReturnToForm(0);
                    return true;
                }
            }
#endif
            PluginLaunched = 0;  
            // Update the driver list and trigger in the UI. 
#define GetObjectPtr(frm, id) FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, id))
            list = GetObjectPtr(form, ctlID_PluginListDialog_PluginList);
            for (i = 0; i < MAXPLUGIN; i++) {
                if (PluginTable[i].dbRef) {
                    listName[i] = PluginTable[i].name;
                    count++;
                } else {
                    listName[i] = "----";
                    count++;
                }                
            }
            LstSetListChoices(list, listName, count);
            if (Environement.screen == PLUGINTYPE_ABOUTBOX)
                FrmShowObject(form, FrmGetObjectIndex(form, ctlID_PluginListDialog_DeleteButton));
            FrmDrawForm( form);
        return true;
        case frmCloseEvent:
            form = FrmGetActiveForm(); 
            list = GetObjectPtr(form, ctlID_PluginListDialog_PluginList);
            LstSetListChoices(list, NULL, 0);
        return false;
        case ctlSelectEvent: 
            switch( event->data.ctlSelect.controlID)
            {
            case ctlID_PluginListDialog_StartButton:
                form = FrmGetActiveForm(); 
                list = GetObjectPtr(form, ctlID_PluginListDialog_PluginList); 

                PluginLaunched = LstGetSelection( list);
                FrmReturnToForm(0);
                if( PluginLaunched != noListSelection && PluginTable[PluginLaunched].dbRef)
                    Plug_Execute(PluginLaunched, Environement, selector);
            return true;
            case ctlID_PluginListDialog_CancelButton:
                FrmReturnToForm(0);
            return true;
            case ctlID_PluginListDialog_DeleteButton:
                form = FrmGetActiveForm(); 
                list = GetObjectPtr(form, ctlID_PluginListDialog_PluginList); 

                PluginLaunched = LstGetSelection( list);
                FrmReturnToForm(0);
                if (PluginLaunched != noListSelection && PluginTable[PluginLaunched].dbRef) {
                    LocalID dbID;
                    UInt16 cardNo, count;
                        
                    DmOpenDatabaseInfo(PluginTable[PluginLaunched].dbRef, &dbID, &count, NULL, &cardNo, NULL);
                    if (count == 1) {
                        Restore(PluginLaunched);
                        DmDeleteDatabase(cardNo, dbID);
                    }
                }
            return true;
            } 
        default:
        break;
        }
#undef GetObjectPtr
        return false;

}

Err Plug_Setup(void)
{
    Err error = errNone;
    Boolean start = true;
    DmSearchStateType state;
    UInt16 cardNo;
    LocalID dbID;
    int i;
    SysLibEntryProcPtr theStarter;

    MemSet(&state, sizeof(DmSearchStateType), 0);
    for (i = 0; i < MAXPLUGIN; i++) {
        PluginTable[i].dbRef = 0;
        PluginTable[i].shLibRef = 0;
        cardNo = 0;
        dbID = 0;

        if (DmGetNextDatabaseByTypeCreator(start, &state, 
                LibType, PlugLibCreator, false, &cardNo, &dbID) == errNone) {

            DmDatabaseInfo( cardNo, dbID, PluginTable[i].name, NULL, NULL, NULL, NULL, NULL,
                    NULL, NULL, NULL, NULL, NULL);
            PluginTable[i].dbRef = DmOpenDatabase( cardNo, dbID, dmModeReadOnly);
            if (!PluginTable[i].dbRef) {
                error = -1;
                break;
            }

            PluginTable[i].resH = DmGetResource( 'libr', 0);

            // Make sure we found it
            if (!PluginTable[i].resH) {
                DmCloseDatabase(PluginTable[i].dbRef);
                PluginTable[i].dbRef = 0;
                error = -1;
                break;
            }

            error = SysLibFind (PluginTable[i].name, &(PluginTable[i].shLibRef));
            if (error)  {
                // Lock down the plug-in so it doesn't move
                theStarter = (SysLibEntryProcPtr) LockMemHandle( PluginTable[i].resH);
                error = SysLibInstall(theStarter, &(PluginTable[i].shLibRef));
                UnlockMemHandle( PluginTable[i].resH);
            }

            if(error || !PluginTable[i].shLibRef) {
                DmReleaseResource( PluginTable[i].resH);
                DmCloseDatabase(PluginTable[i].dbRef);
                PluginTable[i].dbRef = 0;
                break;
            }
            error = PlugLibOpen(PluginTable[i].shLibRef);
            if(error) {
                SysLibRemove(PluginTable[i].shLibRef);
                DmReleaseResource( PluginTable[i].resH);
                DmCloseDatabase(PluginTable[i].dbRef);
                PluginTable[i].dbRef = 0;
                break;
            }
        } else
            break;
        start = false;
    }
    return error;
}

static void Restore(int i)
{
    Err error;
    Int16 usecount;

    error = PlugLibClose(PluginTable[i].shLibRef, &usecount);

    if (usecount == 0)
        SysLibRemove(PluginTable[i].shLibRef);

    // Release the plug-in resource
    DmReleaseResource( PluginTable[i].resH);

    DmCloseDatabase(PluginTable[i].dbRef);
    PluginTable[i].dbRef = 0;
}

void Plug_Restore(void)
{
    int i = 0;

    for (i = 0; i < MAXPLUGIN; i++) {
        if (PluginTable[i].dbRef)
            Restore(i);
    }
}

DataSourceDriver *Plug_GetDriver(void)
{
    int i = 0;
    DataSourceDriver * driverList, * driverCurrent;

    driverList = driverCurrent = NULL;
    for (i = 0; i < MAXPLUGIN; i++) {
        if (PluginTable[i].dbRef) {
            DataSourceDriver * driver;
            driver = PlugLib_getDriver(PluginTable[i].shLibRef);
            if (driver){
                if (driverList == NULL)
                    driverList = driverCurrent = driver;
                else {
                    driverCurrent->next = driver;
                    driverCurrent = driverCurrent->next;
                }
            }
        }
    }
        
    return driverList;
}

void Plug_ShowList( PlugEnvironement environement)
{
    MemMove(&Environement, &environement, sizeof(Environement));
    FrmPopupForm( formID_PluginListDialog);
}

void Plug_ShowAboutBox()
{
    Environement.screen = PLUGINTYPE_ABOUTBOX;
    FrmPopupForm( formID_PluginListDialog);
}    

void Plug_AutoRun( PlugEnvironement environement)
{
    UInt16 i = 0;
    Switcher type = autorun;

    if (environement.moment == FORM_CLOSE)
        type = autorunclose;

    while (i < MAXPLUGIN) {
        if (PluginTable[i].dbRef)
            Plug_Execute(i, environement, type);
        i++;
    }

}


Boolean Plug_CurrentPluginHandleEvent( EventPtr event)
{
    return false;
}

GridType * Plug_GetGrid(void)
{
    int i;

    for (i = 0; i < MAXPLUGIN; i++) {
        if (PluginTable[i].dbRef) {
            GridType * grid;
            if (PlugLib_getVersion(PluginTable[i].shLibRef) < 3)
                continue;
            grid = PlugLib_getGrid(PluginTable[i].shLibRef);
            if (grid){
                return grid;
            }
        }
    }
    return NULL;
}
